package net.meutel.recettes.api.service;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.stereotype.Service;

import net.meutel.recettes.api.exception.ItemNotFoundException;
import net.meutel.recettes.api.mapper.ReceipeFullMapper;
import net.meutel.recettes.api.mapper.ReceipeSimplifiedMapper;
import net.meutel.recettes.api.model.Receipe;
import net.meutel.recettes.api.repository.ReceipeRepository;

@Service
public class ReceipeServiceImpl implements ReceipeService {

    private final ReceipeRepository repo;
    private final ReceipeSimplifiedMapper mapper;
    private final ReceipeFullMapper fullMapper;

    public ReceipeServiceImpl(ReceipeRepository repo, ReceipeSimplifiedMapper mapper, ReceipeFullMapper fullMapper) {
        this.repo = repo;
        this.mapper = mapper;
        this.fullMapper = fullMapper;
    }

    @Override
    public List<Receipe> findReceipes() {
        return repo.findAll()
                .stream()
                .map(mapper::fromEntity)
                .collect(Collectors.toList());
    }

    @Override
    public Receipe getReceipeById(String id) {
        return repo.findById(id)
                .map(fullMapper::fromEntity)
                .orElseThrow(() -> new ItemNotFoundException("Receipe", id));
    }

}
