package net.meutel.recettes.api.controller;

import static java.util.stream.Collectors.toList;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.hasSize;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.stream.Stream;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.oauth2Login;

import net.meutel.recettes.api.model.RecetteParam;
import net.meutel.recettes.api.service.ParametersService;

@SpringBootTest
@AutoConfigureMockMvc
public class ParameterControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private ParametersService service;

    @Test
    public void listRecetteParamsByType_noAuth_302() throws Exception {
        this.mockMvc.perform(get("/parameters/TEST"))
                .andExpect(status().is3xxRedirection());
    }

    @Test
    public void listRecetteParamsByType_any_emptyArray() throws Exception {
        this.mockMvc.perform(get("/parameters/TEST").with(oauth2Login()))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$").isArray())
                .andExpect(jsonPath("$").isEmpty());
    }

    @Test
    public void listRecetteParamsByType_exists_array() throws Exception {
        when(service.loadAllParameters("TEST"))
                .thenReturn(Stream.of("p1", "p2")
                        .map(n -> new RecetteParam().name(n).atType("TEST").id(n))
                        .collect(toList()));

        this.mockMvc.perform(get("/parameters/TEST").with(oauth2Login()))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$").isArray())
                .andExpect(jsonPath("$").isNotEmpty())
                .andExpect(jsonPath("$", hasSize(2)))
                .andExpect(jsonPath("$.[*].name").exists())
                .andExpect(jsonPath("$.[*].id").exists())
                .andExpect(jsonPath("$.[*].@type").exists())
                .andExpect(jsonPath("$.[1].@type", equalTo("TEST")))
                .andExpect(jsonPath("$.[0].name", equalTo("p1")));
    }
}