package net.meutel.recettes.api.service;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

import java.util.List;

import net.meutel.recettes.api.entity.RecetteParamEntity;
import net.meutel.recettes.api.model.RecetteParam;
import net.meutel.recettes.api.repository.RecetteParamRepository;

@SpringBootTest
public class ParametersServiceImplTest {

        @MockBean
        RecetteParamRepository repo;

        @Autowired
        ParametersService tested;

        @Test
        void loadAllParameters_exists_list() {
                var p1 = new RecetteParamEntity();
                p1.setId("1");
                p1.set_type("TEST");
                p1.setName("p1");
                var p2 = new RecetteParamEntity();
                p2.setId("2");
                p2.set_type("TEST");
                p2.setName("p2");

                when(repo.findBy_type("TEST"))
                                .thenReturn(List.of(p1, p2).stream());

                var result = tested.loadAllParameters("TEST");

                assertThat(result)
                                .isNotNull()
                                .hasSize(2);
                assertThat(result).element(0)
                                .returns("TEST", RecetteParam::getAtType)
                                .returns("p1", RecetteParam::getName);
                assertThat(result).element(1)
                                .returns("TEST", RecetteParam::getAtType)
                                .returns("p2", RecetteParam::getName);
        }

        @Test
        void loadAllParameters_notExists_empty() {
                var result = tested.loadAllParameters("TEST");

                assertThat(result)
                                .isNotNull()
                                .isEmpty();
        }

}
